﻿using Newtonsoft.Json;
using SFXamIssues.Models.Http;
using System;
using System.IO;
using System.Net.Http;
using System.Threading;
using System.Threading.Tasks;

namespace SFXamIssues.Extensions
{
    public static class HttpClientExtensions
    {

        public static HttpRequestMessage GenerateMessage(this HttpClient httpClient)
        {
            var httpRequestMessage = new HttpRequestMessage()
            {
                Method = HttpMethod.Get
            };

            httpRequestMessage.Headers.Accept.Clear();
            httpRequestMessage.Headers.Accept.Add(new System.Net.Http.Headers.MediaTypeWithQualityHeaderValue("application/json"));
            httpRequestMessage.Content.Headers.ContentType = new System.Net.Http.Headers.MediaTypeHeaderValue("application/json");

            return httpRequestMessage;
        }

        /// <summary>
        /// Generic method to perform a web request and return a response.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="httpClient"></param>
        /// <param name="httpRequestMessage"></param>
        /// <returns></returns>
        public static async Task<HttpResponse<T>> MakeRequest<T>(this HttpClient httpClient, HttpRequestMessage httpRequestMessage, CancellationToken cancellationToken)
        {


            HttpResponse<T> response = new HttpResponse<T>();

            try
            {
                var webResponse = await httpClient.SendAsync(httpRequestMessage);

                webResponse.EnsureSuccessStatusCode();

                var readResponse = await webResponse.Content.ReadAsStreamAsync();

                response.Result = readResponse.FromJsonStream<T>();
                response.Success = true;
            }
            catch (Exception ex)
            {
                response.Exception = ex.ToString();
                response.Success = false;
            }

            return response;

        }


        public static async Task<HttpResponse> MakeRequest(this HttpClient httpClient, HttpRequestMessage httpRequestMessage, CancellationToken cancellationToken)
        {

            var response = new HttpResponse();

            try
            {
                var webResponse = await httpClient.SendAsync(httpRequestMessage);

                webResponse.EnsureSuccessStatusCode();

                var readResponse = await webResponse.Content.ReadAsStreamAsync();

                response.Success = true;
            }
            catch (Exception ex)
            {
                response.Exception = ex.ToString();
                response.Success = false;
            }

            return response;

        }

        public static T FromJsonStream<T>(this Stream stream)
        {
            if (stream == null)
            {
                throw new ArgumentNullException(nameof(stream));
            }

            if (!stream.CanRead)
            {
                throw new NotSupportedException("Can't read from this stream.");
            }

            using (var streamReader = new StreamReader(stream))
            {
                using (var jsonTextReader = new JsonTextReader(streamReader))
                {
                    var jsonSerializer = new JsonSerializer();
                    return jsonSerializer.Deserialize<T>(jsonTextReader);
                }
            }
        }

    }
}
