﻿using Prism.Navigation;
using SFXamIssues.Extensions;
using SFXamIssues.Models;
using SFXamIssues.Models.Http;
using SFXamIssues.Models.Movies;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading;
using System.Threading.Tasks;

namespace SFXamIssues.ViewModels
{
    public class MainPageViewModel : ViewModelBase
    {

        #region "Web client"
        
        public readonly Uri BaseAddress = new Uri("https://bootcommoney.co.uk");


        public async Task<HttpResponse<T>> MakeRequest<T>(HttpRequestMessage httpRequestMessage, CancellationToken cancellationToken)
        {

            using (var handler = new HttpClientHandler())
            {
#if DEBUG
                handler.ServerCertificateCustomValidationCallback =
                    (message, cert, chain, errors) => { return true; };
#endif

                handler.AutomaticDecompression = System.Net.DecompressionMethods.GZip;

                var httpClient = HttpClientFactory.Create(handler);
                httpClient.Timeout = new TimeSpan(0, 0, 30);
                httpClient.DefaultRequestHeaders.Clear();

                var response = await httpClient.MakeRequest<T>(httpRequestMessage, cancellationToken);
                return response;
            }

        }

        public HttpRequest CreateRequestMessage(Uri uri, HttpMethod method)
        {
            var result = new HttpRequest();
            result.RequestMessage = new HttpRequestMessage(method, uri);
            result.TokenSource = new CancellationTokenSource();
            result.RequestMessage.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
            result.RequestMessage.Headers.AcceptEncoding.Add(new StringWithQualityHeaderValue("gzip"));
            return result;
        }

        #endregion

        #region "Lookup calls"

        private ObservableCollection<Models.Movies.MovieRating> _movieRatingLookupList;

        public ObservableCollection<Models.Movies.MovieRating> MovieRatingLookupList { get { return _movieRatingLookupList; } set { _movieRatingLookupList = value; RaisePropertyChanged(nameof(MovieRatingLookupList)); } }

        private ObservableCollection<Models.Movies.MovieRegion> _movieRegionLookupList;

        public ObservableCollection<Models.Movies.MovieRegion> MovieRegionLookupList { get { return _movieRegionLookupList; } set { _movieRegionLookupList = value; RaisePropertyChanged(nameof(MovieRegionLookupList)); } }

        public async Task LoadMovieLookups()
        {
            var movieRatingRequestMessage = CreateRequestMessage(new Uri($"{BaseAddress.ToString()}MovieLookups/Ratings"), HttpMethod.Get);
            var movieRatingResponse = await MakeRequest<List<MovieRating>>(movieRatingRequestMessage.RequestMessage, movieRatingRequestMessage.TokenSource.Token);
            if (movieRatingResponse.Success)
            {
                MovieRatingLookupList = new ObservableCollection<MovieRating>(movieRatingResponse.Result);
            }

            var movieRegionRequestMessage = CreateRequestMessage(new Uri($"{BaseAddress.ToString()}MovieLookups/Regions"), HttpMethod.Get);
            var movieRegionResponse = await MakeRequest<List<MovieRegion>>(movieRegionRequestMessage.RequestMessage, movieRegionRequestMessage.TokenSource.Token);
            if (movieRegionResponse.Success)
            {
                MovieRegionLookupList = new ObservableCollection<MovieRegion>(movieRegionResponse.Result);
            }

        }

        private DataRecord _dataSource;

        public DataRecord DataSource { get { return _dataSource; } set { _dataSource = value; RaisePropertyChanged(nameof(DataSource)); } }

        #endregion
        public MainPageViewModel(INavigationService navigationService)
            : base(navigationService)
        {
            Title = "Main Page";
            this.DataSource = new DataRecord { Rating = 81, Region = 9 };
            LoadMovieLookups();
        }
    }
}
